/*******************************************************************************
**
** Source File Name = rechist.c  %I%
**
** Licensed Materials - Property of IBM
**
** (C) COPYRIGHT International Business Machines Corp. 1995
** All Rights Reserved.
**
** US Government Users Restricted Rights - Use, duplication or
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
**
**
**    PURPOSE :
**       an example showing how to use RECOVERY HISTORY FILE
**       SCAN APIs in order to:
**          - Update a Recovery History File
**          - List the contents of a Recovery History File
**          - Prune the Recovery History File
**
**    APIs USED :
**      BACKUP DATABASE                         sqlubkup
**      OPEN RECOVERY HISTORY FILE SCAN         sqluhops
**      GET NEXT RECOVERY HISTORY FILE ENTRY    sqluhgne
**      CLOSE RECOVERY HISTORY FILE SCAN        sqluhcls
**      UPDATE RECOVERY HISTORY FILE            sqluhupd
**      PRUNE RECOVERY HISTORY FILE             sqluhprn
**
**    STRUCTURES USED :
**       sqlca
**
**    OTHER FUNCTIONS DECLARED :
**       'C' COMPILER LIBRARY :
**          stdio.h  -  printf
**
**       internal :
**          list_rhf :         Displays a directory of databases
**
**       external :
**          check_error :     Checks for SQLCODE error, and prints out any
**          [in UTIL.C]          related information available.
**                               This procedure is located in the UTIL.C file.
**
**    EXTERNAL DEPENDENCIES :
**       - Existing database for precompile purposes.
**       - Precompile with the SQL precompiler (PREP in DB2)
**       - Binding to a database (BIND in DB2)
**       - Compiling and linking with the IBM Cset++ compiler (AIX and OS/2)
**         or the Microsoft Visual C++ compiler (Windows)
**         or the compiler supported on your platform.
**
**       NOTE: To run this example, the DB2DBDFT environment variable
**             must be set to "sample".
**
*******************************************************************************/
/*--> SQLB0X18.C */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sqlutil.h>
#include <sqlenv.h>
#include "util.h"
/*<-- */

#define  CHECKERR(CE_STR)   if (check_error (CE_STR, &sqlca) != 0) return 1;

int list_rhf (struct sqlca, char *, char *);

int main (int argc, char *argv[]) {
   struct sqlca sqlca;

   /* Variables for the BACKUP DATABASE API */
   struct sqlu_media_list mediaList;
   char applid[SQLU_APPLID_LEN+1];

/*--> */
   char timestamp[SQLU_TIME_STAMP_LEN+1];
   char objectPart[SQLU_TIME_STAMP_LEN+4];
/*<-- */

   char location[SQLU_DB_DIR_LEN+1];
   unsigned short forceOption;


   if (argc != 4) {
      printf ("\nUSAGE: rechist dbname userid passwd\n\n");
      return 1;
   } /* endif */
   printf( "Sample C program: RECHIST.C\n" );



   mediaList.target.media = (struct sqlu_media_entry *) malloc (sizeof(struct sqlu_media_entry));
   mediaList.media_type = 'L';
   mediaList.sessions   = 1;

   #if (defined(DB2AIX) || defined(DB2SUN) || defined(DB2HP) \
	|| defined(DB2SNI) || defined(DB2SCO) || defined(DB2SGI))
   strcpy (mediaList.target.media->media_entry, "/dev/null");
   #else
   strcpy (mediaList.target.media->media_entry, "NUL");
   #endif

   printf ("creating a history file entry ... backing up the database to null\n");
   /*****************\
   * BACKUP DATABASE *
   \*****************/
   sqlubkup (argv[1], 0, 0, 0, 0, applid, timestamp, 0, NULL, &mediaList,
      argv[2], argv[3], NULL, 0, NULL, NULL, &sqlca);
   CHECKERR ("BACKUP DATABASE");

   strcpy(objectPart, timestamp);
   strcat(objectPart, "001");
   printf ("object part = :%s:\n", objectPart);

   printf ("updating the recovery history file...\n");
/*--> */
   /******************************\
   * UPDATE RECOVERY HISTORY FILE *
   \******************************/
   sqluhupd (objectPart , NULL, NULL, "THIS IS A COMMENT", NULL, &sqlca);
/*<-- */
   CHECKERR ("TEST UPDATE");
/*   sqluhupd (&objectPart, &newLocation, &newDeviceType, &newComment, NULL,
      &sqlca);
*/
   CHECKERR ("UPDATE RECOVERY HISTORY FILE");

   printf ("listing all entries to the recovery history file...\n");
   list_rhf (sqlca, timestamp, argv[1]);

   printf ("Pruning the recovery history file...\n");
/*--> */
   /*****************************\
   * PRUNE RECOVERY HISTORY FILE *
   \*****************************/
   sqluhprn (timestamp, 1, NULL, &sqlca);
/*<-- */
   CHECKERR ("PRUNE RECOVERY HISTORY FILE");

   printf ("listing all entries to the recovery history file [after PRUNE]\n");
   list_rhf (sqlca, timestamp, argv[1]);

   return 0;
}

/*--> SQLB0X36.C */
/*******************************************************************************
** Procedure : list_rhf
**
** Purpose :   This procedure displays the data contained in each recovery
**             history file entry.
**
*******************************************************************************/
int list_rhf (struct sqlca sqlca, char *tstamp, char *dbname) {
   short idx;
   unsigned short count;
   unsigned short handle;
   unsigned short callerac = 1;   /* for history */
   char *database = "sample";
   char *objectName = "\0";
   struct sqluhinfo *buffer;

   /*********************************\
   * OPEN RECOVERY HISTORY FILE SCAN *
   \*********************************/
   sqluhops (dbname, tstamp, objectName, &count, &handle, callerac,
      NULL, &sqlca);
/*<-- */
   CHECKERR ("OPEN RECOVERY HISTORY FILE SCAN");
   printf ("count = %d\ntimestamp = %s\n", count, tstamp);

   buffer = (struct sqluhinfo *) malloc (SQLUHINFOSIZE(count));
   buffer->sqln = count;

   do {
/*--> */
      /**************************************\
      * GET NEXT RECOVERY HISTORY FILE ENTRY *
      \**************************************/
      sqluhgne (handle, NULL, buffer, &sqlca);
/*<-- */
      if (sqlca.sqlcode == SQLUH_SQLUHINFO_VARS_WARNING) {

         count = buffer->sqld;
         free (buffer);

         buffer = (struct sqluhinfo *) malloc (SQLUHINFOSIZE(count));
         buffer->sqln = count;

      } else if (sqlca.sqlcode == SQLE_RC_NOMORE) {
         break;
      } else {
printf ("need to print out the information from the sqluhinfo struct\n\n");
/* need to print out the buffer information at this point */
         CHECKERR ("GET NEXT RECOVERY HISTORY FILE SCAN");
      } /* endif */
   } while ( 1 ); /* enddo */

/*--> */
   /**********************************\
   * CLOSE RECOVERY HISTORY FILE SCAN *
   \**********************************/
   sqluhcls (handle, NULL, &sqlca);
/*<-- */
   CHECKERR ("CLOSE RECOVERY HISTORY FILE SCAN");
   return 0;
}
