/*
 *  Sample code to perform the following operations using the Call Level
 *  Interface:
 *
 *       - Connect to a database.
 *       - Create a table
 *       - Insert data using parameter markers
 *       - Retrieve the data
 *       - Drop the table
 *       - Disconnect from the database.
 */

#include <sqlcli.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/*
 *  Prototypes.
 */


void print_err(
    SQLHENV hEnv,
    SQLHDBC hDbc,
    SQLHSTMT hStmt );
/*
 *  Parameters.
 *      argv[1] - database name.
 *      argv[2] - user identifier (optional).
 *      argv[3] - user password (optional).
 */

int main( int argc, char * argv[] )
{
    SQLHENV hEnv = SQL_NULL_HENV;
    SQLHDBC hDbc = SQL_NULL_HDBC;
    SQLHSTMT hStmt = SQL_NULL_HSTMT;
    SQLRETURN rc = SQL_SUCCESS;
    SQLCHAR * pszDSN = NULL;
    SQLCHAR * pszUserId = NULL;
    SQLCHAR * pszPassword = NULL;
    SQLCHAR szCreate[] = "CREATE TABLE CLISAMPL (COL1 VARCHAR(50))";
    SQLCHAR szInsert[] = "INSERT INTO CLISAMPL VALUES (?)";
    SQLCHAR szSelect[] = "SELECT * FROM CLISAMPL";
    SQLCHAR szDrop[] = "DROP TABLE CLISAMPL";
    SQLCHAR szCol[51];
    SQLINTEGER cbCol;
    SQLSMALLINT cCol;
    SQLCHAR szColName[33];
    SQLSMALLINT fSqlType;
    SQLINTEGER cbColDef;
    SQLSMALLINT ibScale;
    SQLSMALLINT fNullable;

/*
 *  Check the number of program parameter. If not equal to 2 (e.g.
 *  pgmname dbname) then abort.
 */

    if( argc < 2 )
    {
        printf( "Usage is: %s dbname [userid password]\n", argv[0] );
        exit( 1 );
    }

/*
 *  Allocate an environment and a connection and then try to connect to
 *  database.
 */


    rc = SQLAllocEnv( &hEnv );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

    rc = SQLAllocConnect( hEnv, &hDbc );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

    pszDSN = argv[1];
    if( argc > 2 )
    {
        pszUserId = argv[2];
    }
    if( argc > 3 )
    {
        pszPassword = argv[3];
    }

    printf( "Connecting\n" );
    rc = SQLConnect(
        hDbc,
        pszDSN,
        SQL_NTS,
        pszUserId,
        SQL_NTS,
        pszPassword,
        SQL_NTS );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

/*
 *  Allocate a statement and create the sample table. This code assumes
 *  that the table CLISAMPL does not exist.
 */

    rc = SQLAllocStmt( hDbc, &hStmt );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

    printf( "Create table - %s\n", szCreate );
    rc = SQLExecDirect( hStmt, szCreate, SQL_NTS );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

/*
 *  Commit the changes.
 */

    rc = SQLTransact( hEnv, hDbc, SQL_COMMIT );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

/*
 *  Prepare an insert statement.
 */

    printf( "Insert - %s\n", szInsert );
    rc = SQLPrepare( hStmt, szInsert, SQL_NTS );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

/*
 *  Set input parameter.
 */

    rc = SQLSetParam(
        hStmt,
        1,
        SQL_C_CHAR,
        SQL_VARCHAR,
        50,
        0,
        szCol,
        &cbCol );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

/*
 *  Insert a couple of rows into the database.
 */

    cbCol = SQL_NTS;
    strcpy( szCol, "Row 1" );
    rc = SQLExecute( hStmt );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

    strcpy( szCol, "Row 2" );
    rc = SQLExecute( hStmt );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

/*
 *  Commit the changes.
 */

    rc = SQLTransact( hEnv, hDbc, SQL_COMMIT );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

/*
 *  Reset input parameter.
 */

    rc = SQLFreeStmt( hStmt, SQL_RESET_PARAMS );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

/*
 *  Execute the select statement.
 */

    printf( "Select - %s\n", szSelect );
    rc = SQLExecDirect( hStmt, szSelect, SQL_NTS );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

/*
 *  Return number of columns and describe result set.
 */

    rc = SQLNumResultCols( hStmt, &cCol );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }
    printf( "Number of columns - %d\n", cCol );

    rc = SQLDescribeCol(
        hStmt,
        1,
        szColName,
        sizeof( szColName ),
        NULL,
        &fSqlType,
        &cbColDef,
        &ibScale,
        &fNullable );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }
    printf( "Column name - %s\n", szColName );
    printf( "Column type - %d\n", fSqlType );
    printf( "Column precision - %ld\n", cbColDef );
    printf( "Column scale - %d\n", ibScale );
    printf( "Column nullable - %s\n", ( fNullable ) ? "TRUE" : "FALSE" );

/*
 *  Bind the output parameter.
 */

    rc = SQLBindCol(
        hStmt,
        1,
        SQL_C_CHAR,
        szCol,
        sizeof( szColName ),
        NULL );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

/*
 *  Fetch data.
 */

    rc = SQLFetch( hStmt );
    if( rc == SQL_ERROR )
    {
        goto exit;
    }
    while( rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO )
    {
        printf( "Column value - %s\n", szCol );
        rc = SQLFetch( hStmt );
        if( rc == SQL_ERROR )
        {
            goto exit;
        }
    }

/*
 *  Close cursor and free bound columns.
 */

    rc = SQLFreeStmt( hStmt, SQL_CLOSE );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

    rc = SQLFreeStmt( hStmt, SQL_UNBIND );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

/*
 *  Drop table.
 */

    rc = SQLExecDirect( hStmt, szDrop, SQL_NTS );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

/*
 *  Commit work.
 */

    rc = SQLTransact( hEnv, hDbc, SQL_COMMIT );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

/*
 *  Disconnect and free up CLI resources.
 */

    printf( "Disconnecting\n" );
    rc = SQLFreeStmt( hStmt, SQL_DROP );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }
    hStmt = SQL_NULL_HSTMT;

    rc = SQLDisconnect( hDbc );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

    rc = SQLFreeConnect( hDbc );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

    rc = SQLFreeConnect( hDbc );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }
    hDbc = SQL_NULL_HDBC;

    rc = SQLFreeEnv( hEnv );
    if( rc != SQL_SUCCESS )
    {
        goto exit;
    }

/*
 *  Print out error if any and exit program.
 */

exit:
    if( rc == SQL_SUCCESS_WITH_INFO || rc == SQL_ERROR )
    {
        print_err( hEnv, hDbc, hStmt );
    }
    printf( "Exiting program\n" );
    return( (int)rc );
}

void print_err(
    SQLHENV hEnv,
    SQLHDBC hDbc,
    SQLHSTMT hStmt )
{
    SQLRETURN rc;
    SQLCHAR szSqlState[SQL_MAX_DSN_LENGTH+1];
    SQLCHAR szErrorMsg[SQL_MAX_MESSAGE_LENGTH+1];
    SQLINTEGER fNativeErr;
    SQLSMALLINT cbErrorMsg;

/*
 *  Retrieve last error.
 */

    rc = SQLError(
        hEnv,
        hDbc,
        hStmt,
        szSqlState,
        &fNativeErr,
        szErrorMsg,
        (SQLSMALLINT)sizeof( szErrorMsg ),
        &cbErrorMsg );
    if( rc != SQL_NO_DATA_FOUND )
    {
        printf( "Error message -\n%s\n", szErrorMsg );
        printf( "Error message length - %d\n", cbErrorMsg );
        printf( "Native error - %ld\n", fNativeErr );
        printf( "SQLState - %s\n", szSqlState );
    }
    return;
}
