/*******************************************************************************
**                                                                        
** Source File Name = compnd.c  %I%                                      
**                                                                        
** Licensed Materials - Property of IBM                                   
**                                                                        
** (C) COPYRIGHT International Business Machines Corp. 1995               
** All Rights Reserved.                                                   
**                                                                        
** US Government Users Restricted Rights - Use, duplication or            
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      
**                                                                        
**                                                                        
**    PURPOSE :                                                           
**    Demonstrates how to execute a compound SQL statement.
**    Creates an 'awards' table and executes a compound statement
**    with 4 insert sub-statemens.
**                                                                        
**                                                                        
**    FUNCTIONS USED :                                                    
**        SQLAllocEnv
**        SQLAllocStmt
**        SQLDisconnect
**        SQLExecDirect
**        SQLExecute
**        SQLFreeConnect
**        SQLFreeEnv
**        SQLFreeStmt
**        SQLPrepare
**                                                                        
**                                                                        
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "sqlcli1.h"
#include "samputil.h"

/* For the Macintosh environment when generating 68K applications */
#ifdef DB268K
   /* Need to include ASLM for 68K applications */
   #include <LibraryManager.h>
#endif


#define MAX_STMT_LEN 255
/* Global Variables for user id and password.
   To keep samples simple, not a recommended practice.
*/
    SQLCHAR   server[SQL_MAX_DSN_LENGTH + 1];
    SQLCHAR   uid[MAX_UID_LENGTH + 1];
    SQLCHAR   pwd[MAX_PWD_LENGTH + 1];

/*******************************************************************
** main
*******************************************************************/
int
main(  int argc, char * argv[] )
{
    SQLHENV         henv;
    SQLHDBC         hdbc;
    SQLRETURN       rc;

/*--> SQLL1X21.SCRIPT */
    SQLHSTMT        hstmt, cmhstmt[4];
    SQLCHAR         compst[4][512] = {
        "INSERT INTO awards (id, award) "
           "SELECT id, 'Sales Merit' from staff "
             "WHERE job = 'Sales' AND (comm/100 > years)",
/**/
        "INSERT INTO awards (id, award) "
           "SELECT id, 'Clerk Merit' from staff "
             "WHERE job = 'Clerk' AND (comm/50 > years)",
/**/
        "INSERT INTO awards (id, award) "
           "SELECT id, 'Best ' concat job FROM STAFF "
              "WHERE comm = (SELECT max(comm) FROM staff WHERE job = 'Clerk')",
/**/
        "INSERT INTO awards (id, award) "
           "SELECT id, 'Best ' concat job FROM STAFF "
        "WHERE comm = (select max(comm) FROM STAFF WHERE job = 'Sales')\0"
        };
/*<-- */

    SQLINTEGER      i;
    SQLSMALLINT     infolen;
    SQLCHAR         compsupp[2] = "N";

#ifdef DB268K
   /* Before making any API calls for 68K environment, need to initialize the
      Library Manager */
   InitLibraryManager(0,kCurrentZone,kNormalMemory);
   atexit(CleanupLibraryManager);
#endif


 /* macro to initalize server, uid and pwd */
    INIT_UID_PWD;

    rc = SQLAllocEnv(&henv);
    if (rc == SQL_ERROR)
        return (terminate(henv, rc));

    rc = DBconnect(henv, &hdbc);/* allocate a connect handle, and connect */
    if (rc == SQL_ERROR)
        return (terminate(henv, rc));

    rc = SQLAllocStmt(hdbc, &hstmt);
    CHECK_DBC(hdbc, rc);

    rc = SQLExecDirect(hstmt, (SQLCHAR *)"DROP TABLE AWARDS", SQL_NTS);
    rc = SQLExecDirect(hstmt,
                       (SQLCHAR *)"CREATE TABLE AWARDS (ID INTEGER, AWARD CHAR(12))",
                       SQL_NTS);
    CHECK_STMT(hstmt, rc);

    rc = SQLTransact(henv, hdbc, SQL_COMMIT);
    CHECK_DBC(hdbc, rc);

/*--> */
    /* Prepare 4 substatements */
    for (i = 1; i < 4; i++) {
        rc = SQLAllocStmt(hdbc, &cmhstmt[i]);
        rc = SQLPrepare(cmhstmt[i], compst[i], SQL_NTS);
        CHECK_STMT(cmhstmt[i], rc);
    }
    rc = SQLExecDirect(hstmt, (SQLCHAR *)"BEGIN COMPOUND NOT ATOMIC STATIC", SQL_NTS);
    CHECK_STMT(hstmt, rc);

    /* Execute 4 substatements */
    for (i = 1; i < 4; i++) {
        rc = SQLExecute(cmhstmt[i]);
        CHECK_STMT(cmhstmt[i], rc);
    }

    /* Execute the COMPOUND statement (of 4 sub-statements)  */
    printf("Executing the COMPOUND statement (of 4 sub-statements)\n");

    rc = SQLExecDirect(hstmt, (SQLCHAR *)"END COMPOUND COMMIT", SQL_NTS);
    CHECK_STMT(hstmt, rc);

    rc = SQLFreeStmt(hstmt, SQL_DROP);
    CHECK_STMT(hstmt, rc);

    for (i = 1; i < 4; i++) {
        rc = SQLFreeStmt(cmhstmt[i], SQL_DROP);
        CHECK_STMT(cmhstmt[i], rc);
    }

/*<-- */
    rc = SQLTransact(henv, hdbc, SQL_COMMIT);
    CHECK_DBC(hdbc, rc);

    printf("Disconnecting .....\n");
    rc = SQLDisconnect(hdbc);
    if (rc != SQL_SUCCESS)
        check_error(henv, hdbc, SQL_NULL_HSTMT, rc, __LINE__, __FILE__);

    rc = SQLFreeConnect(hdbc);
    if (rc != SQL_SUCCESS)
        check_error(henv, hdbc, SQL_NULL_HSTMT, rc, __LINE__, __FILE__);

    rc = SQLFreeEnv(henv);
    if (rc != SQL_SUCCESS)
        terminate(henv, rc);

    return (SQL_SUCCESS);
}                               /* end main */
