/*******************************************************************************
**                                                                        
** Source File Name = drivrcon.c  %I%                                      
**                                                                        
** Licensed Materials - Property of IBM                                   
**                                                                        
** (C) COPYRIGHT International Business Machines Corp. 1995               
** All Rights Reserved.                                                   
**                                                                        
** US Government Users Restricted Rights - Use, duplication or            
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      
**                                                                        
**                                                                        
**    PURPOSE :                                                           
**    - demonstrate basic connection to two datasources using SQLDriverConnect
**    - error handling  ignored for simplicity
**  Refer to multiccon.c for an expanded version.
**                                                                        
**    FUNCTIONS USED :                                                    
**        SQLAllocConnect
**        SQLAllocEnv
**        SQLDisconnect
**        SQLDriverConnect
**        SQLError
**        SQLFreeConnect
**        SQLFreeEnv
**        SQLSetConnectOption
**                                                                        
**                                                                        
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "sqlcli1.h"

/* For the Macintosh environment when generating 68K applications */
#ifdef DB268K
   /* Need to include ASLM for 68K applications */
   #include <LibraryManager.h>
#endif

int
drv_connect(SQLHENV henv,
            SQLHDBC * hdbc,
            SQLCHAR con_type);

SQLRETURN
print_error(SQLHENV henv,
            SQLHDBC hdbc,
            SQLHSTMT hstmt,
            SQLRETURN frc,   /* Return code to be included with error msg  */
            SQLINTEGER line, /* Used for output message, indcate where     */
            SQLCHAR * file);

#define MAX_UID_LENGTH    18
#define MAX_PWD_LENGTH    30
#define MAX_CONNECTIONS   2

SQLRETURN
check_error(SQLHENV henv,
            SQLHDBC hdbc,
            SQLHSTMT hstmt,
            SQLRETURN frc,
            SQLINTEGER line,
            SQLCHAR * file);
int
main()
{
    SQLHENV         henv;
    SQLHDBC         hdbc[MAX_CONNECTIONS];

/* For the Macintosh environment when generating 68K applications */
#ifdef DB268K
   /* Before making any API calls for 68K environment, need to initialize the
      Library Manager */
   InitLibraryManager(0,kCurrentZone,kNormalMemory);
   atexit(CleanupLibraryManager);
#endif

    /* allocate an environment handle   */
    SQLAllocEnv(&henv);

    /* Connect to first data source */
    drv_connect(henv, &hdbc[0], '1');

    /*
     * LONGDATA_COMPAT=1 could have been set on SQLDriverConnect call, shown
     * here as an example of SQLSetConnectOption()
     */
    if (SQLSetConnectOption(hdbc[0], SQL_LONGDATA_COMPAT, SQL_LD_COMPAT_YES) !=
        SQL_SUCCESS) {
        printf(">---ERROR while setting LONGDATA COMPAT ON ------------\n");
    }
    /* Connect to second data source */
    drv_connect(henv, &hdbc[1], '1');

    /*********   Start Processing Step  *************************/
    /* allocate statement handle, execute statement, etc.       */
    /*********   End Processing Step  ***************************/

    printf("\nDisconnecting .....\n");
    SQLDisconnect(hdbc[0]);     /* disconnect first connection */
    SQLDisconnect(hdbc[1]);     /* disconnect second connection */
    SQLFreeConnect(hdbc[0]);    /* free first connection handle  */
    SQLFreeConnect(hdbc[1]);    /* free second connection handle */
    SQLFreeEnv(henv);           /* free environment handle       */

    return (SQL_SUCCESS);
}
/*--> SQLL1X34.SCRIPT */
/********************************************************************
**   drv_connect - Prompt for connect options and connect              **
********************************************************************/

int
drv_connect(SQLHENV henv,
            SQLHDBC * hdbc,
            SQLCHAR con_type)
{
    SQLRETURN       rc;
    SQLCHAR         server[SQL_MAX_DSN_LENGTH + 1];
    SQLCHAR         uid[MAX_UID_LENGTH + 1];
    SQLCHAR         pwd[MAX_PWD_LENGTH + 1];
    SQLCHAR         con_str[255];
    SQLCHAR         buffer[255];
    SQLSMALLINT     outlen;

    printf("Enter Server Name:\n");
    gets((char *) server);
    printf("Enter User Name:\n");
    gets((char *) uid);
    printf("Enter Password Name:\n");
    gets((char *) pwd);

    SQLAllocConnect(henv, hdbc);/* allocate a connection handle     */
    sprintf((char *)con_str, "DSN=%s;UID=%s;PWD=%s;AUTOCOMMIT=0;CONNECTTYPE=1;",
            server, uid, pwd);

    rc = SQLDriverConnect(*hdbc,
    (SQLHWND) NULL,
    con_str,
    SQL_NTS,
    NULL, 0, NULL,
                          SQL_DRIVER_NOPROMPT);
    if (rc != SQL_SUCCESS) {
        printf("Error while connecting to database, RC= %ld\n", rc);
        check_error(SQL_NULL_HENV, *hdbc, SQL_NULL_HSTMT, rc, __LINE__, (SQLCHAR *)__FILE__);
        return (SQL_ERROR);
    } else {
        printf("Successful Connect\n");
        return (SQL_SUCCESS);
    }
}
/*<-- */
/*--> SQLL1X32.SCRIPT */
/*******************************************************************
**  - print_error   - call SQLError(), display SQLSTATE and message
**                  - called by check_error, see below
*******************************************************************/

SQLRETURN
print_error(SQLHENV henv,
            SQLHDBC hdbc,
            SQLHSTMT hstmt,
            SQLRETURN frc,   /* Return code to be included with error msg  */
            SQLINTEGER line, /* Used for output message, indcate where     */
            SQLCHAR * file)  /* the error was reported from  */
{
    SQLCHAR         buffer[SQL_MAX_MESSAGE_LENGTH + 1];
    SQLCHAR         sqlstate[SQL_SQLSTATE_SIZE + 1];
    SQLINTEGER      sqlcode;
    SQLSMALLINT     length;


    printf(">--- ERROR -- RC= %ld Reported from %s, line %ld ------------\n",
           frc, file, line);
    while (SQLError(henv, hdbc, hstmt, sqlstate, &sqlcode, buffer,
                    SQL_MAX_MESSAGE_LENGTH + 1, &length) == SQL_SUCCESS) {
        printf("         SQLSTATE: %s\n", sqlstate);
        printf("Native Error Code: %ld\n", sqlcode);
        printf("%s \n", buffer);
    };
    printf(">--------------------------------------------------\n");
    return (SQL_ERROR);

}

/*******************************************************************
**  - check_error   - call print_error(), checks severity of return code
*******************************************************************/
SQLRETURN
check_error(SQLHENV henv,
            SQLHDBC hdbc,
            SQLHSTMT hstmt,
            SQLRETURN frc,
            SQLINTEGER line,
            SQLCHAR * file)
{
    SQLRETURN       rc;

    print_error(henv, hdbc, hstmt, frc, line, file);

    switch (frc) {
    case SQL_SUCCESS:
        break;
    case SQL_INVALID_HANDLE:
        printf("\n>------ ERROR Invalid Handle --------------------------\n");
    case SQL_ERROR:
        printf("\n>--- FATAL ERROR, Attempting to rollback transaction --\n");
        rc = SQLTransact(henv, hdbc, SQL_ROLLBACK);
        if (rc != SQL_SUCCESS)
            printf(">Rollback Failed, Exiting application\n");
        else
            printf(">Rollback Successful, Exiting application\n");
        exit(0);
        break;
    case SQL_SUCCESS_WITH_INFO:
        printf("\n> ----- Warning Message, application continuing --------\n");
        break;
    case SQL_NO_DATA_FOUND:
        printf("\n> ----- No Data Found, application continuing --------- \n");
        break;
    default:
        printf("\n> ----------- Invalid Return Code --------------------- \n");
        printf("> --------- Attempting to rollback transaction ---------- \n");
        SQLTransact(henv, hdbc, SQL_ROLLBACK);
        exit(0 );
        break;
    }
    return (SQL_SUCCESS);

}
