/*******************************************************************************
**                                                                        
** Source File Name = getsqlca.c  %I%                                      
**                                                                        
** Licensed Materials - Property of IBM                                   
**                                                                        
** (C) COPYRIGHT International Business Machines Corp. 1995               
** All Rights Reserved.                                                   
**                                                                        
** US Government Users Restricted Rights - Use, duplication or            
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      
**                                                                        
**                                                                        
**    PURPOSE :                                                           
** Example of executing interactive SQL statements, displaying result sets
** and simple transaction management.
** This is indentical to adhoc.c, with the addition of estimate information
** for the cost and # of rows to be returned.
** Runstats should be run on the tables accessed in order to receive 
** accurate estimate information.
**                                                                        
**    FUNCTIONS USED :                                                    
**        SQLAllocEnv
**        SQLAllocStmt
**        SQLBindCol
**        SQLColAttributes
**        SQLDescribeCol
**        SQLDisconnect
**        SQLExecute
**        SQLFetch
**        SQLFreeConnect
**        SQLFreeEnv
**        SQLFreeStmt
**        SQLGetSQLCA
**        SQLNumResultCols
**        SQLPrepare
**        SQLRowCount
** Local Functions:
**   process_stmt     display_results
**
** Other functions (from samputil):
**           DBconnect              check_error
**                                                                        
**                                                                        
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "sqlcli1.h"
#include "samputil.h"

/* For the Macintosh environment when generating 68K applications */
#ifdef DB268K
   /* Need to include ASLM for 68K applications */
   #include <LibraryManager.h>
#endif

#define  MAX_STMT_LEN 255
#define  MAXCOLS   100

#ifndef max
#define  max(a,b) (a > b ? a : b)
#endif

/* Global Variables for user id and password.
   To keep samples simple, not a recommended practice.
*/
    SQLCHAR   server[SQL_MAX_DSN_LENGTH + 1];
    SQLCHAR   uid[MAX_UID_LENGTH + 1];
    SQLCHAR   pwd[MAX_PWD_LENGTH + 1];

/* Function declarations: */
int display_results(SQLHSTMT hstmt, SQLSMALLINT nresultcols);
int process_stmt(SQLHENV henv, SQLHDBC hdbc, SQLCHAR * sqlstr);

/*******************************************************************
** main
** - initialize
** - start a transaction
**   - get statement
**   - another statement?
** - COMMIT or ROLLBACK
** - another transaction?
** - terminate
*******************************************************************/
int
main( int argc, char * argv[] )
{
    SQLHENV         henv;
    SQLHDBC         hdbc;
    SQLCHAR         sqlstmt[MAX_STMT_LEN + 1] = "";
    SQLCHAR         sqltrans[sizeof("ROLLBACK")];
    SQLRETURN       rc;

/* For the Macintosh environment when generating 68K applications */
#ifdef DB268K
   /* Before making any API calls for 68K environment, need to initialize the
      Library Manager */
   InitLibraryManager(0,kCurrentZone,kNormalMemory);
   atexit(CleanupLibraryManager);
#endif

 /* macro to initalize server, uid and pwd */
    INIT_UID_PWD;

    rc = SQLAllocEnv(&henv);    /* allocate an environment handle    */
    if (rc != SQL_SUCCESS)
        return (terminate(henv, rc));

    rc = DBconnect(henv, &hdbc);/* allocate a connect handle, and connect */
    if (rc != SQL_SUCCESS)
        return (terminate(henv, rc));

    printf("Enter an SQL statement to start a transaction(or 'q' to Quit):\n");
    gets((char *)sqlstmt);

    while (sqlstmt[0] != 'q') {
        while (sqlstmt[0] != 'q') {
            rc = process_stmt(henv, hdbc, sqlstmt);
            if (rc == SQL_ERROR)
                return (SQL_ERROR);
            printf("Enter an SQL statement(or 'q' to Quit):\n");
            gets((char *)sqlstmt);
        }

        printf("Enter 'c' to COMMIT or 'r' to ROLLBACK the transaction\n");
        fgets((char *)sqltrans, sizeof("ROLLBACK"), stdin);

        if (sqltrans[0] == 'c') {
            rc = SQLTransact(henv, hdbc, SQL_COMMIT);
            if (rc == SQL_SUCCESS)
                printf("Transaction commit was successful\n");
            else
                check_error(henv, hdbc, SQL_NULL_HSTMT, rc, __LINE__, __FILE__);
        }
        if (sqltrans[0] == 'r') {
            rc = SQLTransact(henv, hdbc, SQL_ROLLBACK);
            if (rc == SQL_SUCCESS)
                printf("Transaction roll back was successful\n");
            else
                check_error(henv, hdbc, SQL_NULL_HSTMT, rc, __LINE__, __FILE__);
        }
        printf("Enter an SQL statement to start a transaction or 'q' to quit\n");
        gets((char *)sqlstmt);
    }

    printf("Disconnecting .....\n");
    rc = SQLDisconnect(hdbc);
    CHECK_DBC(hdbc, rc);

    rc = SQLFreeConnect(hdbc);
    CHECK_DBC(hdbc, rc);

    rc = SQLFreeEnv(henv);
    if (rc != SQL_SUCCESS)
        return (terminate(henv, rc));

    return (SQL_SUCCESS);
}                               /* end main */

/*******************************************************************
** process_stmt
** - allocates a statement handle
** - executes the statement
** - Gets SQLCA, prompts to continue
** - determines the type of statement
**   - if there are no result columns, therefore non-select statement
**      - if rowcount > 0, assume statement was UPDATE, INSERT, DELETE
**     else
**      - assume a DDL, or Grant/Revoke statement
**   else
**      - must be a select statement.
**      - display results
** - frees the statement handle
*******************************************************************/

int
process_stmt(SQLHENV henv,
             SQLHDBC hdbc,
             SQLCHAR * sqlstr)
{
    SQLHSTMT        hstmt;
    SQLSMALLINT     nresultcols;
    SQLINTEGER      rowcount;
    SQLRETURN       rc;
    struct sqlca    sqlca;
    SQLCHAR         prompt[255];

    SQLAllocStmt(hdbc, &hstmt); /* allocate a statement handle */

/*--> SQLL1X15.SCRIPT */
    /* execute the SQL statement in "sqlstr"    */

    rc = SQLPrepare(hstmt, sqlstr, SQL_NTS);
    CHECK_STMT(hstmt, rc);

    rc = SQLGetSQLCA(henv, hdbc, hstmt, &sqlca);
    CHECK_STMT(hstmt, rc);

    printf(" Relative Cost=[%ld] Estimated # rows=[%ld]\n"
           " Continue with execution(Y or N)?\n",
           sqlca.sqlerrd[3], sqlca.sqlerrd[2]);
    gets((char *)prompt);
    if (prompt[0] == 'n' || prompt[0] =='N')
        return(0);

    if (rc != SQL_SUCCESS)
        check_error(henv, hdbc, hstmt, rc, __LINE__, __FILE__);
    rc = SQLExecute(hstmt);
/*<-- */

    if (rc != SQL_SUCCESS)
        if (rc == SQL_NO_DATA_FOUND) {
            printf("\nStatement executed without error, however,\n");
            printf("no data was found or modified\n");
            return (SQL_SUCCESS);
        } else
            check_error(henv, hdbc, hstmt, rc, __LINE__, __FILE__);

    rc = SQLNumResultCols(hstmt, &nresultcols);
    CHECK_STMT(hstmt, rc);

    /* determine statement type */
    if (nresultcols == 0) {     /* statement is not a select statement */
        rc = SQLRowCount(hstmt, &rowcount);
        if (rowcount > 0) {     /* assume statement is UPDATE, INSERT, DELETE */
            printf("Statement executed, %ld rows affected\n", rowcount);
        } else {                /* assume statement is GRANT, REVOKE or a DLL
                                 * statement */
            printf("Statement completed successful\n");
        }
    } else {                    /* display the result set */
        display_results(hstmt, nresultcols);
    }                           /* end determine statement type */

    rc = SQLFreeStmt(hstmt, SQL_DROP); /* free statement handle */
    CHECK_STMT(hstmt, rc);

    return (0);
}                               /* end process_stmt */

/*******************************************************************
** display_results
**
**  - for each column
**      - get column name
**      - bind column
**  - display column headings
**  - fetch each row
**      - if value truncated, build error message
**      - if column null, set value to "NULL"
**      - display row
**      - print truncation message
**  - free local storage
*******************************************************************/
display_results(SQLHSTMT hstmt,
                SQLSMALLINT nresultcols)
{
    SQLCHAR         colname[32];
    SQLSMALLINT     coltype;
    SQLSMALLINT     colnamelen;
    SQLSMALLINT     nullable;
    SQLUINTEGER     collen[MAXCOLS];
    SQLSMALLINT     scale;
    SQLINTEGER      outlen[MAXCOLS];
    SQLCHAR        *data[MAXCOLS];
    SQLCHAR         errmsg[256];
    SQLRETURN       rc;
    int             i;
    SQLINTEGER      displaysize;

    for (i = 0; i < nresultcols; i++) {
        SQLDescribeCol(hstmt, i + 1, colname, sizeof(colname),
                       &colnamelen, &coltype, &collen[i], &scale, NULL);

        /* get display length for column */
        SQLColAttributes(hstmt, i + 1, SQL_COLUMN_DISPLAY_SIZE, NULL, 0,
                         NULL, &displaysize);

        /*
         * set column length to max of display length, and column name
         * length.  Plus one byte for null terminator
         */
        collen[i] = max(displaysize, strlen((char *) colname)) + 1;

        printf("%-*.*s", (int)collen[i], (int)collen[i], colname);

        /* allocate memory to bind column                             */
        data[i] = (SQLCHAR *) malloc((int)collen[i]);

        /* bind columns to program vars, converting all types to CHAR */
        rc = SQLBindCol(hstmt, i + 1, SQL_C_CHAR, data[i], collen[i], &outlen[i]);
    }
    printf("\n");

    /* display result rows                                            */
    while ((rc = SQLFetch(hstmt)) != SQL_NO_DATA_FOUND) {
        errmsg[0] = '\0';
        for (i = 0; i < nresultcols; i++) {
            /* Check for NULL data */
            if (outlen[i] == SQL_NULL_DATA)
                printf("%-*.*s", (int)collen[i], (int)collen[i], "NULL");
            else
            {   /* Build a truncation message for any columns truncated */
                if (outlen[i] >= collen[i]) {
                    sprintf((char *) errmsg + strlen((char *) errmsg),
                            "%d chars truncated, col %d\n",
                            (int)outlen[i] - collen[i] + 1, i + 1);
                }
                /* Print column */
                printf("%-*.*s", (int)collen[i], (int)collen[i], data[i]);
             }
        }                       /* for all columns in this row  */

        printf("\n%s", errmsg); /* print any truncation messages    */
    }                           /* while rows to fetch */

    /* free data buffers                                              */
    for (i = 0; i < nresultcols; i++) {
        free(data[i]);
    }

}                               /* end display_results */
