/*******************************************************************************
**                                                                        
** Source File Name = ordrep.c  %I%                                      
**                                                                        
** Licensed Materials - Property of IBM                                   
**                                                                        
** (C) COPYRIGHT International Business Machines Corp. 1995               
** All Rights Reserved.                                                   
**                                                                        
** US Government Users Restricted Rights - Use, duplication or            
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      
**                                                                        
**                                                                        
**    PURPOSE :                                                           
** - Uses Array Input and Array Output to generate multiple result sets.
**   Each result set is one order.                                        
**
**    FUNCTIONS USED :                                                    
**        SQLAllocEnv
**        SQLAllocStmt
**        SQLBindCol
**        SQLBindParameter
**        SQLDisconnect
**        SQLExecDirect
**        SQLExtendedFetch
**        SQLFreeConnect
**        SQLFreeEnv
**        SQLMoreResults
**        SQLParamOptions
**        SQLSetStmtOption
**                                                                        
**                                                                        
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "sqlcli.h"
#include "sqlcli1.h"
#include "samputil.h"           /* Header file for CLI sample code */

/* For the Macintosh environment when generating 68K applications */
#ifdef DB268K
   /* Need to include ASLM for 68K applications */
   #include <LibraryManager.h>
#endif

#define MAX_STMT_LEN 255
#define ROWSET_SIZE  100

/* Global Variables for user id and password.
   To keep samples simple, not a recommended practice.
*/
    SQLCHAR   server[SQL_MAX_DSN_LENGTH + 1];
    SQLCHAR   uid[MAX_UID_LENGTH + 1];
    SQLCHAR   pwd[MAX_PWD_LENGTH + 1];

/*******************************************************************
** main
*******************************************************************/
int
main( int argc, char * argv[] )
{
    SQLHENV         henv;
    SQLHDBC         hdbc;
    SQLHSTMT        hstmt;
    SQLRETURN       rc;

/*--> SQLL1X26.SCRIPT */
    SQLCHAR         stmt[] =
    {  "WITH "  /* Common Table expression (or Define Inline View) */
         "order (ord_num, cust_num, prod_num, quantity, amount) AS "
         "( "
         "SELECT c.ord_num, c.cust_num, l.prod_num, l.quantity,  "
                 "price(char(p.price, '.'), p.units, char(l.quantity, '.')) "
            "FROM ord_cust c, ord_line l, product p  "
            "WHERE c.ord_num = l.ord_num AND l.prod_num = p.prod_num  "
             "AND cast (cust_num as integer) = ? "
         "), "
         "totals (ord_num, total) AS "
         "( "
          "SELECT ord_num, sum(decimal(amount, 10, 2))  "
          "FROM order GROUP BY ord_num "
         ") "
       /* The 'actual' SELECT from the inline view */
       "SELECT order.ord_num, cust_num, prod_num, quantity,  "
              "DECIMAL(amount,10,2) amount, total "
        "FROM order, totals  "
        "WHERE order.ord_num = totals.ord_num "
     };

    /* Array of customers to get list of all orders for */
    SQLINTEGER     Cust[]=
    {
        10, 20, 30, 40, 50, 60, 70, 80, 90, 100,
        110, 120, 130, 140, 150, 160, 170, 180, 190, 200,
        210, 220, 230, 240, 250
    };

#define  NUM_CUSTOMERS sizeof(Cust)/sizeof(SQLINTEGER)

    /* Row-Wise (Includes buffer for both column data and length) */
    struct {
        SQLINTEGER      Ord_Num_L;
        SQLINTEGER      Ord_Num;
        SQLINTEGER      Cust_Num_L;
        SQLINTEGER      Cust_Num;
        SQLINTEGER      Prod_Num_L;
        SQLINTEGER      Prod_Num;
        SQLINTEGER      Quant_L;
        SQLDOUBLE       Quant;
        SQLINTEGER      Amount_L;
        SQLDOUBLE       Amount;
        SQLINTEGER      Total_L;
        SQLDOUBLE       Total;
    }               Ord[ROWSET_SIZE];

    SQLUINTEGER     pirow = 0;
    SQLUINTEGER     pcrow;
    SQLINTEGER      i;
    SQLINTEGER      j;
/*<-- */

/* For the Macintosh environment when generating 68K applications */
#ifdef DB268K
   /* Before making any API calls for 68K environment, need to initialize the
      Library Manager */
   InitLibraryManager(0,kCurrentZone,kNormalMemory);
   atexit(CleanupLibraryManager);
#endif

/* macro to initalize server, uid and pwd */
    INIT_UID_PWD;

    rc = SQLAllocEnv(&henv);    /* allocate an environment handle    */
    if (rc != SQL_SUCCESS)
        return (terminate(henv, rc));

    rc = DBconnect(henv, &hdbc);/* allocate a connect handle, and connect */
    if (rc != SQL_SUCCESS)
        return (terminate(henv, rc));

/*--> */
    /* Get details and total for each order Row-Wise */
    rc = SQLAllocStmt(hdbc, &hstmt);
    CHECK_DBC(hdbc, rc);

    rc = SQLParamOptions(hstmt, NUM_CUSTOMERS, &pirow);
    CHECK_STMT(hstmt, rc);

    rc = SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_LONG, SQL_INTEGER,
                          0, 0, Cust, 0, NULL);
    CHECK_STMT(hstmt, rc);

    rc = SQLExecDirect(hstmt, stmt, SQL_NTS);
    CHECK_STMT(hstmt, rc);

    /* SQL_ROWSET_SIZE sets the max number of result rows to fetch each time */
    rc = SQLSetStmtOption(hstmt, SQL_ROWSET_SIZE, ROWSET_SIZE);
    CHECK_STMT(hstmt, rc);

    /* Set Size of One row, Used for Row-Wise Binding Only */
    rc = SQLSetStmtOption(hstmt, SQL_BIND_TYPE, sizeof(Ord) / ROWSET_SIZE);
    CHECK_STMT(hstmt, rc);

    /* Bind column 1 to the Ord_num Field of the first row in the array*/ 
    rc = SQLBindCol(hstmt, 1, SQL_C_LONG, (SQLPOINTER) & Ord[0].Ord_Num, 0,
                    &Ord[0].Ord_Num_L);
    CHECK_STMT(hstmt, rc);

    /* Bind remaining columns ... */
/*<-- */
    rc = SQLBindCol(hstmt, 2, SQL_C_LONG, (SQLPOINTER) & Ord[0].Cust_Num, 0,
                    &Ord[0].Cust_Num_L);
    CHECK_STMT(hstmt, rc);

    rc = SQLBindCol(hstmt, 3, SQL_C_LONG, (SQLPOINTER) & Ord[0].Prod_Num, 0,
                    &Ord[0].Prod_Num_L);
    CHECK_STMT(hstmt, rc);

    rc = SQLBindCol(hstmt, 4, SQL_C_DOUBLE, (SQLPOINTER) & Ord[0].Quant, 0,
                    &Ord[0].Quant_L);
    CHECK_STMT(hstmt, rc);

    rc = SQLBindCol(hstmt, 5, SQL_C_DOUBLE, (SQLPOINTER) & Ord[0].Amount, 0,
                    &Ord[0].Amount_L);
    CHECK_STMT(hstmt, rc);

    rc = SQLBindCol(hstmt, 6, SQL_C_DOUBLE, (SQLPOINTER) & Ord[0].Total, 0,
                    &Ord[0].Total_L);
    CHECK_STMT(hstmt, rc);
/*--> */

    /* NOTE: This sample assumes that an order will never have more
             rows than ROWSET_SIZE.  A check should be added below to call
             SQLExtendedFetch multiple times for each result set.
    */           

    do  /* for each result set .... */
    { rc = SQLExtendedFetch(hstmt, SQL_FETCH_NEXT, 0, &pcrow, NULL);
      if ( rc != SQL_NO_DATA_FOUND ) CHECK_STMT(hstmt, rc);

      if (pcrow > 0) /* if 1 or more rows in the result set */
      {
        i = j = 0;
        
        printf("**************************************\n");
        printf("Orders for Customer: %ld\n", Ord[0].Cust_Num);
        printf("**************************************\n");
       
        while (i < pcrow) 
        {   printf("\nOrder #: %ld\n", Ord[i].Ord_Num);
            printf("     Product  Quantity         Price\n");
            printf("     -------- ---------------- ------------\n");
            j = i;
            while (Ord[j].Ord_Num == Ord[i].Ord_Num) 
            {   printf("    %8ld %16.7lf %12.2lf\n",
                       Ord[i].Prod_Num, Ord[i].Quant, Ord[i].Amount);
                i++;
            }
            printf("                               ============\n");
            printf("                              %12.2lf\n", Ord[j].Total);
        } /* end while */
      } /* end if */
    }
    while ( SQLMoreResults(hstmt) == SQL_SUCCESS);
/*<-- */
	

    if (rc != SQL_NO_DATA_FOUND && rc != SQL_SUCCESS)
        check_error(henv, hdbc, hstmt, rc, __LINE__, __FILE__);

    rc = SQLTransact(henv, hdbc, SQL_COMMIT);
    CHECK_DBC(hdbc, rc);

    printf("Disconnecting .....\n");
    rc = SQLDisconnect(hdbc);
    CHECK_DBC(hdbc, rc);

    rc = SQLFreeConnect(hdbc);
    CHECK_DBC(hdbc, rc);

    rc = SQLFreeEnv(henv);
    if (rc != SQL_SUCCESS)
        return (terminate(henv, rc));

    return (SQL_SUCCESS);
}                               /* end main */
